/* read a file in atari stad pac format and output a pbm
   of size 640x400 pixel */

#include <stdio.h>
#include <string.h>
#include <errno.h>
#include <stdlib.h>

#define byte unsigned char

static byte head[7];
#define pacesc head[4]
#define pacmax head[5]
#define pacmin head[6]

void helper(char *n)
{
  fprintf(stderr,
  "Usage: %s [<file.pac> [<file.pbm>]]\n"
  "convert picture in atari stad format.PAC to pbm 640x400\n",
  n);
  exit(1);
}

void unknown(void)
{
  fprintf(stderr,"Unknown file type %02x%02x%02x%02x\n",
    head[0],head[1],head[2],head[3]);
}

void pm8x(FILE *fi, FILE *fo, int sx, int sy)
{
  int l, a, u, x, v, c;
  byte data [256];
  byte dest [80*400];
  const byte *const hpbm = "P4 640 400\n";
  a = 0;
  u = 0;
  x = 0;
  l = 80*400;
  while (l > 0) {
#ifdef DDEBUG
    fprintf(stderr,"C (l=%5d, a=%3d, u=%3d)\n",l,a,u);
#endif
    if (a < 3) {
      if (u > 0) {
        switch (a) {
          case  2: data[0] = data[u];
                   data[1] = data[u+1];
                   break;
          case  1: data[0] = data[u];
                   break;
          default: break;
        }
        u = 0;
      }
      a += fread(&data[a],1,sizeof(data)-a,fi);
#ifdef DDEBUG
      fprintf(stderr,"R (         a=%3d, u=%3d)\n",a,u);
#endif
    }
    if (a > 0) {
      if (data[u] == pacesc) {
        if (a > 1) {
          c = data[u+1];
          if (c == 0) {
            v = pacesc;
            c = 1;
          } else {
            v = pacmax;
            c += 1;
          }
#ifdef DDEBUG
          fprintf(stderr,"ESC (esc=%02x)",pacesc);
#endif
          a -= 2;
          u += 2;
        } else {
          fprintf(stderr,"Conversion error\n");
          exit(1);
        }
      } else if (data[u] == pacmin) {
        if (a > 2) {
          v = data[u+1];
          c = data[u+2] + 1;
#ifdef DDEBUG
          fprintf(stderr,"MIN (min=%02x)",pacmin);
#endif
          a -= 3;
          u += 3;
        } else {
          fprintf(stderr,"Conversion error\n");
          exit(1);
        }
      } else {
        v = data[u];
        c = 1;
#ifdef DDEBUG
        fprintf(stderr,"MOV (    %02x)",data[u]);
#endif
        a -= 1;
        u += 1;
      }
    } else {
      fprintf(stderr,"Conversion error\n");
      exit(1);
    }
#ifdef DDEBUG
    fprintf(stderr,", (v=%02x,c=%02x,x=%3d,y=%3d)\n",v,c,x%80,x/80);
#endif
    while ((c > 0) && (l > 0)) {
      dest[x] = v;
      x += sx;
      if (x >= 80*400) {
        x += sy;
      }
#ifdef DDEBUG_X
    fprintf(stderr,"XY (%5d;x=%3d,y=%3d)\n",x,x%80,x/80);
#endif
      l--;
      c--;
    }
  }
#ifdef DDEBUG
  fprintf(stderr,"\n");
#endif
  fwrite(hpbm,11,1,fo);
  fwrite(&dest[0],80,400,fo);
}

int main (int argc, char *argv[])
{
  FILE *fi;
  FILE *fo;
  if (argc > 3) {
    fprintf(stderr,"Wrong number of arguments\n");
    helper(argv[0]);
  }
  if ((argc > 1) && (!strcmp(argv[1],"--help"))) {
    helper(argv[0]);
  }
  if (argc > 1) {
    if ((fi = fopen(argv[1],"r")) == NULL) {
      fprintf(stderr,"cannot open %s, %s\n",argv[1],strerror(errno));
      exit(1);
    }
  } else {
    fi = stdin;
  }
  if (argc > 2) {
    if ((fo = fopen(argv[2],"w")) == NULL) {
      fprintf(stderr,"cannot open %s, %s\n",argv[1],strerror(errno));
      exit(1);
    }
  } else {
    fo = stdout;
  }
  if (fread(&head,sizeof(head),1,fi) != 1) {
    fprintf(stderr,"read error, %s",strerror(errno));
    exit(1);
  }
  if ((head[0] == 'p') && (head[1] == 'M') && (head[2] == '8')) {
    if (head[3] == '5') {
      pm8x(fi,fo,1,0);
    } else if (head[3] == '6') {
      pm8x(fi,fo,80,1-80*400);
    } else {
      unknown();
    }
  } else {
    unknown();
  }
  return 0;
}

